using System;
using System.IO;
using System.Xml;
using System.Windows.Forms;

namespace UPSLT
{
	public class cpRaceRunner: System.Windows.Forms.Form
	{
		private System.Windows.Forms.ColumnHeader colName;
		private System.Windows.Forms.ColumnHeader colTime;
		private System.Windows.Forms.ColumnHeader colAge;
		private System.Windows.Forms.ColumnHeader colRank;
		private System.Windows.Forms.ListView lstRunners;
		
		// Private Member Variables
		private Runner[] m_runners;
		private Runner[] m_30plus;
		private Runner[] m_16to29;
		private Runner[] m_0to15;
		
		// Make a mess
		public cpRaceRunner()
		{
			// VS.NET Generated Method -- Don't touch
			InitializeComponent();
			
			// Try to read the data file
			if (ReadData(Application.StartupPath + @"\data\raceresults.xml"))
			{	
				// If ReadData() was successful, then we have 3 arrays with data.
				// We'll QuickSort each array, so we can get the rank of each runner (within his age group)							
				QuickSort(m_0to15, 0, m_0to15.Length - 1, true);									 
				QuickSort(m_16to29, 0, m_16to29.Length - 1, true);
				QuickSort(m_30plus, 0, m_30plus.Length - 1, true);				
				
				// Now copy the 3 arrays into 1 array
				int numRunners = m_0to15.Length + m_16to29.Length + m_30plus.Length;				
				m_runners = new Runner[numRunners];
				Array.Copy(m_0to15, 0, m_runners, 0, m_0to15.Length);
				Array.Copy(m_16to29, 0, m_runners, m_0to15.Length, m_16to29.Length);
				Array.Copy(m_30plus, 0, m_runners, m_0to15.Length + m_16to29.Length, m_30plus.Length);
								
				// Finally, sort the newly created array
				QuickSort(m_runners, 0, m_runners.Length - 1, false); 
				
				// Fill in the List View so we can see the results
				FillListView();
			}
			else
				MessageBox.Show("Invalid data file");
		}

		// Clean up
		protected override void Dispose( bool disposing )
		{
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{			
			this.lstRunners = new System.Windows.Forms.ListView();
			this.colName = new System.Windows.Forms.ColumnHeader();
			this.colTime = new System.Windows.Forms.ColumnHeader();
			this.colAge = new System.Windows.Forms.ColumnHeader();
			this.colRank = new System.Windows.Forms.ColumnHeader();
			this.SuspendLayout();
			// 
			// lstRunners
			// 
			this.lstRunners.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																																								 this.colName,
																																								 this.colTime,
																																								 this.colAge,
																																								 this.colRank});
			this.lstRunners.Location = new System.Drawing.Point(18, 15);
			this.lstRunners.Name = "lstRunners";
			this.lstRunners.Size = new System.Drawing.Size(578, 319);
			this.lstRunners.TabIndex = 0;
			this.lstRunners.View = System.Windows.Forms.View.Details;
			// 
			// colName
			// 
			this.colName.Text = "Runner\'s Name";
			this.colName.Width = 178;
			// 
			// colTime
			// 
			this.colTime.Text = "Time";
			this.colTime.Width = 109;
			// 
			// colAge
			// 
			this.colAge.Text = "Age";
			this.colAge.Width = 88;
			// 
			// colRank
			// 
			this.colRank.Text = "Rank (within age group)";
			this.colRank.Width = 199;
			// 
			// cpRaceRunner
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(614, 355);
			this.Controls.Add(this.lstRunners);
			this.Name = "cpRaceRunner";
			this.Text = "Race Results";
			this.ResumeLayout(false);

		}
		#endregion

		// Main point of entry
		[STAThread]
		static void Main() 
		{
			Application.Run(new cpRaceRunner());
		}
		
		// Private Member Functions
		private bool ReadData(string _datafile)
		{
			bool bResult;
			
			FileInfo fi = new FileInfo(_datafile);
			if (!fi.Exists)
				bResult = false;
			else
			{
				// Read the file
				XmlDocument doc = new XmlDocument();
				try
				{
					doc.Load(_datafile);
					XmlElement root = doc.DocumentElement;
					string qry0to15 = "//RaceResults/Runner[Age<=15]";
					string qry16to29 = "//RaceResults/Runner[Age>=16 and Age<=29]";
					string qry30plus = "//RaceResults/Runner[Age>=30]";
					
					// Query all runners 15 years old or less, and place them in an array
					XmlNodeList nodeList = root.SelectNodes(qry0to15);					
					FillArray(ref m_0to15, nodeList);					
					
					// Query all runners 16 to 29, and place them in an array 
					nodeList = root.SelectNodes(qry16to29);
					FillArray(ref m_16to29, nodeList);
					
					// Query all runners 30 years old or greater, and place them in an array 
					nodeList = root.SelectNodes(qry30plus);
					FillArray(ref m_30plus, nodeList);					

					// ReadData completed successfully
					bResult = true;
				}
				catch (Exception)
				{
					// Mal-formed XML
					bResult = false;
				}				
			}		
			
			// Whatever bResult is at this point, return it.
			return bResult;		
		}
		
		private void FillArray(ref Runner[] runnerArray, XmlNodeList nodeList)
		{
			// Fill in the array (passed by reference) based on the information in the XmlNodeList
			
			int i = 0;
			runnerArray = new Runner[nodeList.Count];			
			foreach (XmlNode n in nodeList)
			{
				string strName = n.SelectSingleNode("Name").InnerText;
				int nTime = Int32.Parse(n.SelectSingleNode("Time").InnerText);
				int nAge = Int32.Parse(n.SelectSingleNode("Age").InnerText);						
				runnerArray[i] = new Runner(strName, nTime, nAge);										
				i++;				
			}			
		}		
		
		// QuickSort Algorithm -- I'm not explaining this
		private void QuickSort(Runner[] runners, int left, int right, bool bRank)
		{
			Runner pivot;
			int lhold, rhold;			
			
			lhold = left;
			rhold = right;
			pivot = runners[left];
			
			int pivotTime = pivot.Time;
			
			while (left < right)
			{
				while ((runners[right].Time >= pivotTime) && (left < right))
					right--;
					
				if (left != right)
				{
					runners[left] = runners[right];					
					left++;
				}
				
				while ((runners[left].Time <= pivotTime) && (left < right))
					left++;
					
				if (left != right)
				{					
					runners[right] = runners[left];
					right--;
				}
			}
			
			runners[left] = pivot;
			pivotTime = left;
			if (bRank)
				pivot.Rank = left + 1;
			left = lhold;
			right = rhold;
			if (left < pivotTime)
				QuickSort(runners, left, pivotTime -  1, bRank);
			if (right > pivotTime)
				QuickSort(runners, pivotTime + 1, right, bRank);
		}
		
		
		private void FillListView()
		{
			for (int i = 0; i < m_runners.Length; i++)
			{
				ListViewItem li = new ListViewItem(m_runners[i].Name);
				li.SubItems.Add(m_runners[i].Time.ToString());
				li.SubItems.Add(m_runners[i].Age.ToString());
				li.SubItems.Add(m_runners[i].Rank.ToString());
				lstRunners.Items.Add(li);
			}
		}
	}
}
