// polycalc.cpp source file

/*
Steve Stchur - COSC336 (Assignment #4)
10/26/00

--Purpose--
The purpose of this program is to implement a linked list technique to add
evaluate polynomials.

--Algorithm / Concept--
I have created a polynomial class which is contained in the files "polynomial.h"
and "polynomial.cpp".  The most important functions are outlined below:

  GetEval(int x)					// returns the value of the polynomial for any given int, x
  GetPolyAsString()					// returns a string representatio of the polynomial
  InsertTerm(int _coeff, int _exp)	// inserts a term Cx^E into the polynomial
  operator +(polynomial &pol)		// allows for the addition of two polynomials

--Important Notes--
The polynomial class is actually a doubly linked list even though the assignment only required
that we use a singly linked list.  The reason for this is because I found it easier to code
the polynomial in order of ascending exponents.  I then use the convienence of the doubly linked
list to traverse the list in reverse order and display the polynomial in order of decreasing
exponents.  

Also, I have used the stl's string type to make string concatenation easier.  However when
displaying a polynomial using cout, it is necessary to convert the string to a "c" style string
using the stl function c_str().

--Input / Output--
I have created a menu to aid the user in entering terms into each of two polynomials, p1 and p2.
The user will see the results of p1 + p2 instantly, each time he adds a term to either
polynomial.

Try the following dataset to test the PolyCalc.Exe program:
- Chose menu option 1 (insert term into p1)
-- Insert 2x^3 by entering 2 3 <return>

- Chose menu option 2 (insert term into p2)
-- Insert 5x^3 by entering 5 3 <return>

You should now see the sum of p1 + p2 = 7x^3

- Chose menu option 1 (insert term into p1)
-- Insert 9 by entering 9 0 <return>

- Chose menu option 2 (insert term into p2)
-- Change the term 5x^3 to 8x^3 by entering 8 3 <return>

You should now see the sum of p1 + p2 = 10x^3 + 9

- Chose menu option 3 (evaluate p1)
-- Evaluate p1 when x = 2 by entering 2 <return>

You should now see (when x = 2) the value of p1 = 25

- Chose menu option 4 (evaluate p2)
-- Evaluate p2 when x = 3 by entering 3 <return>

You should now see (when x = 3) the value of p2 = 216

- Chose menu option 5 (evaluate p1 + p2)
-- Evaluate p1 + p2 when x = 4 by entering 4 <return>

You should now see (when x = 4) the value of p1 + p2 = 649

- Chose menu option 6 (to quit PolyCalc)

*/

#include <iostream.h>				// cin and cout
#include "polynomial.h"				// polynomial data type

int DisplayMenu(polynomial _p1, polynomial _p2);	// DisplayMenu() forward declaration

int main()		
{
	// polynomials p1 and p2 will be used throughout the program as addens in the Poly Calculator
	polynomial p1, p2;
	
	int nUserInput;		// stores the menu option the user has chosen
	int coeff, exp;		// stores the coeff, exp the user enters for each term
	int x;				// stores the x value to which a polynomial is evaluated
	char dummyChar;		// dummy char to pause the program
	
	// while the user has not selected menu option 6 (Quit)
	while ((nUserInput = DisplayMenu(p1, p2)) != 6)	
	{
		// determine which option was selected
		switch(nUserInput)
		{
			// the user has chosen to insert a term into p1
			// so we cin the coeff and exp and insert them into p1.
			// at the 'break' statement, the switch statement exits and
			// the PolyCalc is refreshed (via the while loop)
			case 1:
			{
				cout << "Enter the (coeff exp) pair in the form: coeff <space> exp\n";
				cout << "Example: 4 3 for 4x^3, -3 2 for -3x^2, 5 0 for 5, etc...\n";
				cout << "(coeff exp): ";
				cin >> coeff >> exp;
				p1.InsertTerm(coeff, exp);
				break;
			}

			// the user has chosen to insert a term into p2
			// so we cin the coeff and exp and insert them into p2.
			// PolyCalc is then refreshed (via the while loop)
			case 2:
			{
				cout << "Enter the (coeff exp) pair in the form: coeff <space> exp\n";
				cout << "Example: 4 3 for 4x^3, -3 2 for -3x^2, 5 0 for 5, etc...\n";
				cout << "(coeff exp): ";
				cin >> coeff >> exp;
				p2.InsertTerm(coeff, exp);
				break;
			}

			// the user has chosen to evaluate p1 so we
			// cin the value for x and call GetEval(x) on p1.
			// PolyCalc is then refreshed (via the while loop)
			case 3:
			{
				cout << "What is the value of x? ";
				cin >> x;
				cout << "\nWhen x = " << x << ", p1 = " << p1.GetEval(x) << endl;
				cout << "Press 'c' followed by <return> to continue: ";
				cin >> dummyChar;
				break;
			}

			// the user has chosen to evaluate p2 so we
			// cin the value for x and call GetEval(x) on p2.
			// PolyCalc is then refreshed (via the while loop)
			case 4:
			{
				cout << "What is the value of x? ";
				cin >> x;
				cout << "\nWhen x = " << x << ", p2 = " << p2.GetEval(x) << endl;
				cout << "Press 'c' followed by <return> to continue: ";
				cin >> dummyChar;
				break;
			}

			// the user has chosen to evaluate p1 + p2 so we
			// cin the value for x and call GetEval(x) on p1+p2.
			// PolyCalc is then refreshed (via the while loop)
			case 5:
			{
				cout << "What is the value of x? ";
				cin >> x;
				cout << "\nWhen x = " << x << ", p1 + p2 = " << (p1 + p2).GetEval(x) << endl;
				cout << "Press 'c' followed by <return> to continue: ";
				cin >> dummyChar;
				break;
			}

			// the user has chosen an invalid menu option in which case the
			// PolyCalc simply ignores the input and refreshes,
			// or the user has chosen to quit in which case we do nothing
			// while loop exits
			default:
			case 6:
				// quit PolyCalc
			break;
		}
	}

	return 0;			// program terminated normally
}


// DisplayMenu(polynomial, polynomial) function
// --------------------------------------------
// This function simplys the current values of p1, p2 and p1 + p2.
// Below the values of the polynomials, it display the menu options to the user
// The cin function is used to grab the user's selection into the variable, nSelection
// which then becomes the return value for the function
int DisplayMenu(polynomial _p1, polynomial _p2)
{
	system("cls");
	cout << "*********************************************\n";
	cout << "Welcome to: PolyCalc! (Polynomial Calculator)\n";
	cout << "*********************************************\n\n";
	cout << "Calculator's Current Contents:\n";
	cout << "------------------------------\n";
	cout << "p1      = " << _p1.GetPolyAsString().c_str() << endl;
	cout << "p2      = " << _p2.GetPolyAsString().c_str() << endl;
	cout << "p1 + p2 = " << (_p1 + _p2).GetPolyAsString().c_str() << endl << endl;
	
	cout << "Please make a selection:\n";
	cout << "1) Insert / change a term in polynomial p1\n";
	cout << "2) Insert / change a term in polynomial p2\n";
	cout << "3) Evaluate polynomial p1 for any given integer, x\n";
	cout << "4) Evaluate polynomial p2 for any given integer, x\n";
	cout << "5) Evaluate the sum of p1 + p2 for any given integer, x\n";
	cout << "6) Quit PolyCalc\n\n";
	
	cout << "Your selection: ";

	int nSelection;
	cin >> nSelection;

	return nSelection;
}
	