// Graph.cpp: implementation of the Graph class.
//
//////////////////////////////////////////////////////////////////////

#include "Graph.h"
#include <iostream.h>
#include <fstream.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Graph::Graph()
{
	int i, j;

	// initialize the adjacency matrix to all 0's
	for (i = 0; i < NUM_STATES; i++)
		for (j = 0; j < NUM_STATES; j++)
			m_matrix[i][j] = 0;

	// initialize the visited array to all false
	for (i = 0; i < NUM_STATES; i++)
		m_visited[i] = false;

}

Graph::~Graph()
{
	// nothing to destruct really
}

void Graph::PrintStateName(int state)
{
	// simply prints out the string representation of the 
	// enum type, us_state
	switch(state)
	{
		case 0:
			cout << "Washington\n";
			break;
		case 1:
			cout << "Oregon\n";
			break;
		case 2:
			cout << "California\n";
			break;
		case 3:
			cout << "Nevada\n";
			break;
		case 4:
			cout << "Arizona\n";
			break;
		case 5:
			cout << "Utah\n";
			break;
		case 6:
			cout << "Idaho\n";
			break;
		case 7:
			cout << "Montana\n";
			break;
		case 8:
			cout << "Wyoming\n";
			break;
		case 9:
			cout << "Colorado\n";
			break;
		case 10:
			cout << "New Mexico\n";
			break;
		case 11:
			cout << "Texas\n";
			break;
		case 12:
			cout << "Oklahoma\n";
			break;
		case 13:
			cout << "Kansas\n";
			break;
		case 14:
			cout << "Nebraska\n";
			break;
		case 15:
			cout << "South Dakota\n";
			break;
		case 16:
			cout << "North Dakota\n";
			break;
		case 17:
			cout << "Minnesota\n";
			break;
		case 18:
			cout << "Iowa\n";
			break;
		case 19:
			cout << "Missouri\n";
			break;
		case 20:
			cout << "Arkansas\n";
			break;
		case 21:
			cout << "Louisiana\n";
			break;
		case 22:
			cout << "Mississippi\n";
			break;
		case 23:
			cout << "Alabama\n";
			break;
		case 24:
			cout << "Tennesee\n";
			break;
		case 25:
			cout << "Kentucky\n";
			break;
		case 26:
			cout << "Illinois\n";
			break;
		case 27:
			cout << "Wisconsin\n";
			break;
		case 28:
			cout << "Michigan\n";
			break;
		case 29:
			cout << "Indiana\n";
			break;
		case 30:
			cout << "Ohio\n";
			break;
		case 31:
			cout << "West Virginia\n";
			break;
		case 32:
			cout << "Virginia\n";
			break;
		case 33:
			cout << "North Carolina\n";
			break;
		case 34:
			cout << "South Carolina\n";
			break;
		case 35:
			cout << "Georgia\n";
			break;
		case 36:
			cout << "Florida\n";
			break;
		case 37:
			cout << "Maryland\n";
			break;
		case 38:
			cout << "Deleware\n";
			break;
		case 39:
			cout << "New Jersey\n";
			break;
		case 40:
			cout << "Pennsylvania\n";
			break;
		case 41:
			cout << "New York\n";
			break;
		case 42:
			cout << "Connecticut\n";
			break;
		case 43:
			cout << "Rhode Island\n";
			break;
		case 44:
			cout << "Massachusetts\n";
			break;
		case 45:
			cout << "Vermont\n";
			break;
		case 46:
			cout << "New Hampshire\n";
			break;
		case 47:
			cout << "Maine\n";
			break;
	}

}

void Graph::BuildGraph()
{
	// builds the graph from the file "adjacent_states.txt"

	ifstream ifs;
	ifs.open("adjacent_states.txt");

	int state, i;			// hold the state read in from the file 
							// during each iteration of the loop
	char token = '~';		// when toke = '\n', we've reached newline

	while (!ifs.eof())		// while not end of file
	{
		for (i = 0; i < NUM_STATES; i++)			// loop NUM_STATES times (48)
		{
			while (token != '\n' && !ifs.eof())		// while not newline and not eof
			{
				ifs >> state;						// read in the state
				m_matrix[state][i] = 1;				// mark in the the matrix
				ifs.get(token);						// get the next space
			}
			token = '~';							// when loop exits, reset token
		}
	}

}

void Graph::DFS(int position)
{
	static int count = 0;			// keep count of the number of states
									// so i can pause every 20
	
	PrintStateName(position);		// print the string representation of the state
	count++;						// increment count

	// pause every 20 states for easier readability
	if (count % 20 == 0)			
	{
		char dummy;
		cout << endl;
		cout << "Press any key (followed by return) to continue: ";
		cin >> dummy;
		cout << endl;
	}
	
	m_visited[position] = true;		// mark the state as visited

	// determine which states are adjacent to position
	for (int i = 0; i < NUM_STATES; i++)
	{
		if (m_matrix[position][i] == 1)		// position is adjacent to state i
			if (m_visited[i] == false)		// if not yes visited...
				DFS(i);						// run the DFS function for i
	}
}

void Graph::BFS(int position)
{
	static int count = 0;					// keep count of the states for pausing
	static int level = 1;					// keeps track of the current level
	int levelIndex = position;				// determines if the level needs to be changed
	bool levelFlag = false;					// determines when the levelIndex is updated
	m_queue.push(position);					// push 1st state into queue
	m_visited[position] = true;				// mark it as visited
	
	while (!m_queue.empty())				// while queue isn't empty
	{
		position = m_queue.front();			// grab the top element
		cout << "Level " << level << ": ";
		PrintStateName(position);			// print it to the screen

		// if the current state position = levelIndex
		// then we need to increase the level counter
		// and set levelFlag = true to indicate that
		// a new levelIndex is needed
		if (levelIndex == position)
		{
			level++;
			levelFlag = true;
		}

		m_queue.pop();						// take it out of the queue
		count++;							// increment state count

		// pause every 20 states for easier readability
		if (count % 20 == 0)
		{
			char dummy;
			cout << endl;
			cout << "Press any key (followed by return) to continue: ";
			cin >> dummy;
			cout << endl;
		}

		// determine which states are adjacent to position
		for (int i = 0; i < NUM_STATES; i++)
		{
			if (m_matrix[position][i] == 1)		// position is adjacent to state i
			{
				if (m_visited[i] == false)		// if state i is not yet visited
				{
					m_queue.push(i);			// put it in the queue
					m_visited[i] = true;		// mark it as visited
				}
			}
		}

		// levelFlag == true indicated that a new
		// levelIndex is needed, so we quick check
		// to make sure the queue is not empty (to avoid a crash)
		// and set the levelIndex to the last state in the queue
		// then we reset the levelFlag back to false;
		if (levelFlag) 
		{
			if (!m_queue.empty())
				levelIndex = m_queue.back();
			levelFlag = false;
		}

	}
}

	
// clears the visited states (should be called any graph traversal
void Graph::ClearVisitedStates()
{
	for (int i = 0; i < NUM_STATES; i++)
		m_visited[i] = false;
}
